@@ -12,6 +12,51 @@
 #' example(classified.default)
 classified <- function(x, ...)UseMethod('classified')
 
+#' Create Classified from Factor
+#' 
+#' Creates classified from factor. Uses \code{\link{classified.default}},
+#' but supplies existing levels by default.
+#' 
+#' @export
+#' @return 'classified' 'factor'
+#' @param x see \code{\link{factor}}
+#' @param levels passed to \code{\link{classified.default}}; defaults to \code{levels(x)}
+#' @param labels passed to \code{\link{classified.default}}; must be same length as levels(after removing values in \code{exclude}) and must not contain duplicates
+#' @param exclude see \code{\link{factor}}
+#' @param ordered see \code{\link{factor}}
+#' @param nmax see \code{\link{factor}}
+#' @param ... ignored
+#' @importFrom dplyr distinct
+#' @family classified
+#' @examples
+#' a <- factor(c('c','b','a'))
+#' levels(classified(a))
+#' attr(classified(a), 'codelist')
+classified.factor <- function(
+    x = character(),
+    levels,
+    labels,
+    exclude = NA,
+    ordered = is.ordered(x),
+    nmax = NA,
+    ...
+){
+  if(missing(levels)) levels <- match.fun('levels')(x)
+  levels <- setdiff(levels, exclude)
+  if(missing(labels)) labels <- levels
+  stopifnot(identical(length(levels), length(labels)))
+  if(any(duplicated(labels)))(stop('duplicated labels not supported in this context'))
+  y <- classified.default(
+    x,
+    levels = levels,
+    labels = labels,
+    exclude = exclude,
+    ordered = ordered,
+    nmax = NA,
+    ...
+  )
+  y
+}
 
 #' Create Classified by Default
 #'
@@ -54,7 +99,7 @@ classified <- function(x, ...)UseMethod('classified')
 classified.default <- function(
   x = character(),
   levels,
-  labels = levels,
+  labels,
   exclude = NA,
   ordered = is.ordered(x),
   nmax = NA,
@@ -69,7 +114,8 @@ classified.default <- function(
     bad <- sapply(cl, function(val)val %in% exclude)
     cl <- cl[!bad]
     # mimic non-NA exclude behavior:
-    if(length(exclude) == 0) cl <- c(cl, NA)
+    # @ 0.10.12, commenting next (nonsensical?)
+    # if(length(exclude) == 0) cl <- c(cl, NA)
 
     # default levels and labels
     if(missing(levels)){
@@ -97,6 +143,10 @@ classified.default <- function(
   # should be true using defaults
   if(length(levels) != length(labels))stop('classified requires labels and levels of the same length')
 
+  # under some circumstances, levels has names, which may be NA
+  # then data.frame inherits NA rownames which is an error.
+  names(levels) <- NULL
+  names(labels) <- NULL
   codes <- data.frame(levels = levels, labels = labels)
   if(any(duplicated(codes))){
     duplicated <- anyDuplicated(codes)
@@ -152,7 +202,7 @@ classified.default <- function(
     x = x,
     levels = levels,
     labels = labels,
-    exclude = exclude,
+    exclude = exclude, # but exclusions will have already occurred
     ordered = ordered,
     nmax = nmax
   )
@@ -476,12 +526,13 @@ classified.dvec <- function(
 #'
 #' Coerces classified to integer.
 #' Result is like \code{as.integer(as.numeric(x)) + offset}
-#' but has a guide giving original values. If you need
-#' a simple integer, consider coercing first to numeric.
+#' but has a guide attribute: a list of integers 
+#' whose names are the original levels of x.
+#' If you need a simple integer, consider coercing first to numeric.
 #'
 #' @param x classified, see \code{\link{classified}}
 #' @param offset an integer value to add to intermediate result
-#' @param ... passed to \code{\link{as.numeric}}, code{\link{as.integer}}, and code{\link{desolve}}
+#' @param ... passed to \code{\link{desolve}}
 #' @param persistence whether to return 'dvec' (is.integer(): TRUE) or just integer.
 # @param exclude_attr discard these when preserving attributes of x in result
 #' @export
@@ -507,20 +558,20 @@ classified.dvec <- function(
 #' c('knife','fork','spoon') %>% 
 #'   classified %>%
 #'   as.integer %>% 
-#'   class
+#'   class # integer
 #'   
 #' # remove option to restore default persistence paradigm
 #' options(yamlet_persistence = NULL)
 #' c('knife','fork','spoon') %>% 
 #'   classified %>%
 #'   as.integer %>% 
-#'   class
+#'   class # dvec
 #'   
 #' # locally defeat persistence paradigm
 #' c('knife','fork','spoon') %>% 
 #'   classified %>%
 #'   as.integer(persistence = FALSE) %>% 
-#'   class
+#'   class # integer
 #'   
 #'
 as.integer.classified <- function(
@@ -536,35 +587,52 @@ as.integer.classified <- function(
     as.integer(offset) == offset
   )
   offset <- as.integer(offset)
-  y <- as.numeric(x, ...)
-  y <- as.integer(y, ...) # explicitly casting to int as of 0.9.0
-  y <- y + offset
-  z <- mimic(x, y)
-# r <- unclassified(z)
-  r <- desolve(z, persistence = TRUE, ...) # gives guide instead of codelist at 0.9.0
+  
+  # note: levels(x) should be same as unlist(attr(x, 'codelist'))
+  
+  # y <- as.numeric(x, ...)
+  # y <- as.integer(y, ...) # explicitly casting to int as of 0.9.0
+  # y <- y + offset
+  # z <- mimic(x, y, ...) # drops levels!
+  
+  # x has a codelist and seq gives integer
+  vals <- seq_along(attr(x, 'codelist')) 
+  vals <- vals + offset
+  names(attr(x, 'codelist')) <- vals
+
+  r <- desolve(x, persistence = TRUE, ...) # gives guide instead of codelist at 0.9.0
+  
   # at this point, r should be dvec
   # passing persistence to desolve fails because there is no 
   # vector method for implicit_guide (only a data.frame method)
   if(!persistence) {
     r <- unclass(r)
   }
-  # for(at in names(attributes(x))){
-  #   if(!at %in% exclude_attr){
-  #     attr(r, at) <- attr(x, at)
-  #   }
-  # }
   r
 }
 
 #' Create Classified from Classified
 #'
 #' See \code{\link{classified.default}}.
-#' Currently, calling classified on a
-#' classified object is a non-operation.
+#' Formerly (version 0.10.10), calling classified() on a
+#' classified object was a non-operation.
+#' Currently we call factor(x, ...) and then 
+#' try to reconcile the codelist attribute with resulting 
+#' levels.
+#' 
+#' By default classified is idempotent, such that classified(classified(x)) is
+#' the same as classified(x).  In contrast, factor(factor(x)) will drop unused
+#' levels (not shown). To drop unused levels, use classified(classified(x), drop = TRUE).
 #'
 #' @export
 #' @return 'classified' 'factor'
 #' @param x classified
+#' @param levels passed to \code{\link{factor}}; defaults to \code{levels(x)}
+#' @param labels passed to \code{\link{factor}}; must be same length as levels(after removing values in \code{exclude} and unused levels if \code{drop} is TRUE) and must not contain duplicates
+#' @param exclude passed to \code{\link{factor}}
+#' @param ordered passed to \code{\link{factor}}
+#' @param nmax passed to \code{\link{factor}}
+#' @param drop whether to drop unused levels
 #' @param ... ignored
 #' @keywords internal
 #' @family classified
@@ -579,7 +647,76 @@ as.integer.classified <- function(
 #' classified(b)
 #' identical(b, classified(b))
 
-classified.classified <- function(x, ...)x
+classified.classified <- function(
+    x, 
+    levels,
+    labels,
+    exclude = NULL,
+    ordered = is.ordered(x),
+    nmax = NA,
+    drop = FALSE,
+    ...
+){
+  if(missing(levels)) levels <- match.fun('levels')(x)
+  levels <- setdiff(levels, exclude)
+  if(drop) levels <- levels[levels %in% x]
+  if(missing(labels)) labels <- levels
+  stopifnot(identical(length(levels), length(labels)))
+  if(any(duplicated(labels)))(stop('duplicated labels not supported in this context'))
+  codelist <- attr(x, 'codelist')
+  nms <- names(codelist) # from (character)
+  vals <- as.character(unlist(codelist)) # to (coerced to character)
+  stopifnot(identical(levels(x), vals)) # continuity check: should always be true
+  
+  y <- factor(
+    x, 
+    levels = levels,
+    labels = labels,
+    exclude = exclude,
+    ordered = ordered,
+    nmax = nmax
+  )
+  
+  # now we rebuild the codelist
+  # nms is the original form and order
+  # levels(y) is the current from and order
+  # we need a codelist with levels(y) but names from nms
+  # i.e., we need to (re) map names to the current levels
+  # the current levels though derive from the provided labels
+  # current level order should prevail,
+  # labels should be traced to provided levels,
+  # and thence to provided (codelist) vals,
+  # and thence to provided (codelist) nms
+  
+  codelist <- as.list(type.convert(levels(y), as.is = TRUE))
+  # what provided values of 'levels' match existing values of 'levels', 
+  # which are taken from provided 'labels'?
+  was <- levels[match(levels(y), labels)]
+  # now we have each former level for existing levels(y)
+  # in an order corresponding to levels(y)
+  # Those former levels were necessarily among the vals of former codelist.
+  # we recover the meanings from nms
+  meant <- nms[match(was, vals)]
+  # now we know what these levels meant originally.  Possibly nothing.  Possibly NA.
+  names(codelist) <- meant
+  
+  # all this manipulation could introduce multiple NA as codelist names.
+  # in fact, codelist names should never be duplicated.
+  if(any(duplicated(meant))){
+    example <- meant[duplicated(meant)][[1]]
+    warning('codelist names should not contain duplicates, e.g. ', example)
+  }
+  # enforce attributes
+  nms <- names(attributes(x))
+  nms <- setdiff(nms, c('class','levels','codelist','guide'))
+  for(nm in nms){
+    attr(y, nm) <- attr(x, nm)
+  }
+  
+  attr(y, 'codelist') <- codelist
+  class(y) <- union('classified', class(y))
+  y
+}
 
 # Abbreviate Classified
 # 
